// PostOffice.cpp: implementation of the CPostOffice class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "hostilechat.h"
#include "hostilechatdlg.h"
#include "message.h"
#include "chatdlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPostOffice::CPostOffice()
{
	//   
	InitializeCriticalSection(&m_criticalSection);

	//        
	LPSTR lpszSlot = "\\\\.\\mailslot\\forall";

	m_hForAllMailslot = CreateMailslot( lpszSlot, 0,
		MAILSLOT_WAIT_FOREVER, (LPSECURITY_ATTRIBUTES) NULL);

	if( m_hForAllMailslot == INVALID_HANDLE_VALUE ) {
		AfxMessageBox("   .");
		return;
	}
	//     
	m_hFileForAll = CreateFile("\\\\*\\mailslot\\forall",
		GENERIC_WRITE, FILE_SHARE_READ, (LPSECURITY_ATTRIBUTES) NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, (HANDLE) NULL);

	if( m_hFileForAll == INVALID_HANDLE_VALUE )
	{
		AfxMessageBox("   MSFS.");
		return;
	}

	//       
	lpszSlot = "\\\\.\\mailslot\\hostlechat\\confirm_presense";

	m_hConfirmPresense = CreateMailslot( lpszSlot, 0,
		MAILSLOT_WAIT_FOREVER, (LPSECURITY_ATTRIBUTES) NULL);

	if( m_hConfirmPresense == INVALID_HANDLE_VALUE ) {
		AfxMessageBox("   .");
		return;
	}
}

CPostOffice::~CPostOffice()
{
	if(m_hForAllMailslot != INVALID_HANDLE_VALUE)
		CloseHandle(m_hForAllMailslot);

	if(m_hConfirmPresense != INVALID_HANDLE_VALUE)
		CloseHandle(m_hConfirmPresense);

	if( m_hFileForAll != INVALID_HANDLE_VALUE )
		CloseHandle(m_hFileForAll);

	DeleteCriticalSection(&m_criticalSection);
}

bool CPostOffice::IsValid()
{
	if( m_hForAllMailslot == INVALID_HANDLE_VALUE ||
		m_hConfirmPresense == INVALID_HANDLE_VALUE ||
		m_hFileForAll == INVALID_HANDLE_VALUE )
	{
		return false;
	}
	else return true;
}

bool CPostOffice::Broadcast(CUser user)
{
	CMessage message; message.id = CMessage::MID_CONFIRM_PRESENSE;
	strcpy(message.creator, user.name); 
//	strcpy(message.contents, user.computer);

	return Broadcast(message);
}

bool CPostOffice::Broadcast(CMessage message)
{
	__try {
		EnterCriticalSection(&m_criticalSection);

		if(m_hForAllMailslot == INVALID_HANDLE_VALUE ||
		   m_hFileForAll == INVALID_HANDLE_VALUE) return false;

		DWORD cbWritten;
		DWORD cbMessage, cMessage;
		BOOL fResult;

		cbMessage = cMessage = 0;

		fResult = WriteFile( m_hFileForAll, &message, sizeof(CMessage),
			&cbWritten, (LPOVERLAPPED) NULL);

		if(!fResult)
		{
			AfxMessageBox("   .");
			return false;
		}
	}
	__finally {
		LeaveCriticalSection(&m_criticalSection);
	}

	return true;
}

bool CPostOffice::OnRecieveMessage(CMessage message)
{
    char lpszBuffer[400] = {"\x0"};
	CHostileChatApp *pApp = (CHostileChatApp *)AfxGetApp();


	switch( message.id ) 
	{
	case CMessage::MID_CHAT:
		{
			CChatDlg *pDlg = ((CHostileChatApp* )AfxGetApp())->m_pChatDlg;
			//pDlg->OnRecieveMessage(message);
			message.PostMessage(pDlg, WM_RECIEVE_MESSAGE, 0);
			break;
		}
	case CMessage::MID_IN_CHATROOM:
		{
			//     ChatRoom 
			CHostileChatDlg *pDlg = (CHostileChatDlg *) 
				((CHostileChatApp *)AfxGetApp())->m_pMainWnd;

			//      Online
			message.PostMessage(pDlg, WM_UPDATE_USER, 
				CHostileChatDlg::UPDATE_INCHAT);
		}
		break;
	//       
	case CMessage::MID_CONFIRM_PRESENSE:
		{
			//     Online 
			CHostileChatDlg *pDlg = (CHostileChatDlg *)AfxGetApp()->m_pMainWnd;

			//      Online
			message.PostMessage(pDlg, WM_UPDATE_USER,
				CHostileChatDlg::UPDATE_ONLINE);
		}
		break;
	//     
	case CMessage::MID_BROADCAST:
		{
			CUser user;
			
			pApp->GetCurrentUser(user);

			if(strcmp(user.name, message.addressee) == 0 ||
			    (strcmp(message.addressee, "All") == 0 && 
			     strcmp(user.name, message.creator))) 
			{
				//    IncomingDlg ,   
				//   ,   .
				if( *(pApp->m_pIncomingDlg) != 
					*(pApp->m_pIncomingDlg->GetForegroundWindow()))
				{
					::PostMessage(pApp->m_pTaskBar->GetHWND(), WM_TASKBAR_EVENT, 
						EVENT_INCOMING_MESSAGE, 0);
				}

				message.PostMessage(pApp->m_pIncomingDlg, WM_RECIEVE_MESSAGE, 0);
			}
		}
		break;
	}

	return true;
}

bool CPostOffice::CheckPostStatus()
{
	CMessage lastMessage;

//	Sleep(50);

	__try {
		EnterCriticalSection(&m_criticalSection);

		BOOL fResult;
		DWORD cbRead = 0;
		DWORD cbMessage, cMessage;

		cbMessage = cMessage = 0;
		CMessage message;

		fResult = GetMailslotInfo( m_hForAllMailslot,(LPDWORD) NULL, 
			&cbMessage, // size of next message
			&cMessage, // number of messages
			(LPDWORD) NULL);
	
		if(cbMessage == MAILSLOT_NO_MESSAGE) {
			return false;
		}

		while( cMessage != 0 ) // retrieve all messages
		{
			fResult = ReadFile( m_hForAllMailslot, (LPSTR) &message,
				cbMessage, &cbRead, (LPOVERLAPPED) NULL);

			if(!fResult) {
				AfxMessageBox("can't read mailslot");
				return false;
			}

//		if( message.id == CMessage::MID_CONFIRM_PRESENSE ||
//			message.id == CMessage::MID_IN_CHATROOM) 
//		{
//			OnRecieveMessage(message);
//		}
//		else
			if(	strcmp(lastMessage.addressee, message.addressee) != 0 ||
				strcmp(lastMessage.creator, message.creator) != 0 ||
				strcmp(lastMessage.contents, message.contents) != 0 ||
				(lastMessage.id != message.id)) 
			{
				OnRecieveMessage(message);
			}

			strcpy(lastMessage.addressee, message.addressee);
			strcpy(lastMessage.creator, message.creator);
			strcpy(lastMessage.contents, message.contents);
			lastMessage.id = message.id;
	
			fResult = GetMailslotInfo( m_hForAllMailslot,
					(LPDWORD) NULL, &cbMessage, // size of next message
					&cMessage, // number of messages
					(LPDWORD) NULL);
		}
	}
	__finally {
		LeaveCriticalSection(&m_criticalSection);
	}

	return true;
}

