/////////////////////////////////////////////////////////////////////////
// File:      taskbar.cpp
// Purpose:	  Implements CTaskBarIcon class for manipulating icons on
//            task bar.
// Author:    Julian Smart
// Created:   24/3/98
/////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "taskbar.h"
#include "HostileChat.h"
#include "IconFlashThread.h"

#define _EXPORT __declspec(dllexport)

LONG _EXPORT FAR PASCAL CTaskBarIconWindowProc( HWND hWnd, unsigned msg,
				     UINT wParam, LONG lParam );

char *CTaskBarWindowClass = "CTaskBarWindowClass";

CTaskBarIcon* CTaskBarIcon::sm_first = NULL;
BOOL   CTaskBarIcon::sm_registeredClass = FALSE;
UINT   CTaskBarIcon::sm_taskbarMsg = 0;

CTaskBarIcon::CTaskBarIcon(void)
{
    m_hWnd = 0;
    m_iconAdded = FALSE;
    m_next = NULL;

    AddObject(this);

    if (RegisterWindowClass())
        m_hWnd = CreateTaskBarWindow();

	CWinApp *pApp = AfxGetApp();

	m_hIncomingMessage = pApp->LoadIcon(MAKEINTRESOURCE(IDI_MESSAGE));
	m_hNone = pApp->LoadIcon(MAKEINTRESOURCE(IDI_MESSAGE_NONE));
	m_hMainIcon = pApp->LoadIcon(MAKEINTRESOURCE(IDR_MAINFRAME));
	m_hNewUser = pApp->LoadIcon(MAKEINTRESOURCE(IDI_NEW_USER));

	m_bEventIncomingMessage = false;
	m_pIconFlashThread = NULL;
	m_uiEventType = 0;
}

CTaskBarIcon::~CTaskBarIcon(void)
{
    RemoveObject(this);

    if (m_iconAdded)
    {
        RemoveIcon();
    }

    if (m_hWnd)
    {
        ::DestroyWindow(m_hWnd);
        m_hWnd = 0;
    }
}

// Operations
BOOL CTaskBarIcon::SetIcon(HICON hIcon, const char* tooltip)
{
    if (!IsOK())
        return FALSE;

    NOTIFYICONDATA notifyData;

    memset(&notifyData, 0, sizeof(notifyData));
	notifyData.cbSize = sizeof(notifyData);
	notifyData.hWnd = (HWND) m_hWnd;
	notifyData.uCallbackMessage = sm_taskbarMsg;
	notifyData.uFlags = NIF_MESSAGE ;
	if (hIcon != 0)
		notifyData.uFlags |= NIF_ICON;

    if (((const char*) tooltip != NULL) && (tooltip != ""))
    {
        notifyData.uFlags |= NIF_TIP ;
		lstrcpyn(notifyData.szTip, (char*) tooltip, sizeof(notifyData.szTip));
    }

	notifyData.hIcon = (HICON) hIcon;
    notifyData.uID = 99;

    if (m_iconAdded)
        return Shell_NotifyIcon(NIM_MODIFY, & notifyData);
    else
    {
        m_iconAdded = Shell_NotifyIcon(NIM_ADD, & notifyData);
        return m_iconAdded;
    }
}

BOOL CTaskBarIcon::RemoveIcon(void)
{
    if (!m_iconAdded)
        return FALSE;

    NOTIFYICONDATA notifyData;

    memset(&notifyData, 0, sizeof(notifyData));
	notifyData.cbSize = sizeof(notifyData);
	notifyData.hWnd = (HWND) m_hWnd;
	notifyData.uCallbackMessage = sm_taskbarMsg;
	notifyData.uFlags = NIF_MESSAGE;
	notifyData.hIcon = 0 ; // hIcon;
    notifyData.uID = 99;
    m_iconAdded = FALSE;

    return Shell_NotifyIcon(NIM_DELETE, & notifyData);
}

// Overridables
void CTaskBarIcon::OnMouseMove(void)
{
}

void CTaskBarIcon::OnLButtonDown(void)
{
	CHostileChatApp *pApp = (CHostileChatApp *) AfxGetApp();

	if(m_bEventIncomingMessage) 
	{
		m_pIconFlashThread->KillThread();
		FinishEvent();

		switch(m_uiEventType) 
		{
		case EVENT_INCOMING_MESSAGE:
				//  
				pApp->m_pIncomingDlg->ShowWindow(SW_SHOWNORMAL);
				pApp->m_pIncomingDlg->SetForegroundWindow();
			break;
		default:
			pApp->OnOnline();
		}
	}
	else
		pApp->OnOnline();
}

void CTaskBarIcon::OnLButtonUp(void)
{
}

void CTaskBarIcon::OnRButtonDown(void)
{
	if( !m_bEventIncomingMessage) {
		CWnd *pWnd = AfxGetApp()->m_pMainWnd;

		CMenu menu; menu.LoadMenu(IDR_MAINMENU);

		//       , 
		//      
		CIncomingDlg *pDlg = ((CHostileChatApp *)AfxGetApp())->m_pIncomingDlg;

		if( IsWindow(pDlg->GetSafeHwnd()) ) {
			if( pDlg->m_messageList.GetCount() < 1 ) 
			{
//				UINT id = menu.GetSubMenu(0)->GetMenuItemID(2);
				menu.GetSubMenu(0)->
					EnableMenuItem(ID_RIGHT_VIEWMESSAGES, MF_BYCOMMAND | MF_GRAYED);
			}
		}

		POINT pnt; GetCursorPos(&pnt);

		pWnd->SetForegroundWindow();
		menu.GetSubMenu(0)->TrackPopupMenu( TPM_LEFTALIGN | TPM_RIGHTBUTTON,
			pnt.x, pnt.y, pWnd, NULL);
		PostMessage(m_hWnd, WM_MOUSEMOVE, 0, 0);
	}
	else {
//		OnLButtonDown();
	}
}

void CTaskBarIcon::OnRButtonUp(void)
{
}

void CTaskBarIcon::OnLButtonDClick(void)
{
}

void CTaskBarIcon::OnRButtonDClick(void)
{
}

CTaskBarIcon* CTaskBarIcon::FindObjectForHWND(HWND hWnd)
{
    CTaskBarIcon* obj = sm_first;
    while (obj)
    {
        if (obj->GetHWND() == hWnd)
            return obj;
        obj = obj->m_next;
    }
    return NULL;
}

void CTaskBarIcon::AddObject(CTaskBarIcon* obj)
{
    if (sm_first == NULL)
        sm_first = obj;
    else
    {
        CTaskBarIcon* o = sm_first;
        while (o)
        {
            if (o->m_next == NULL)
            {
                o->m_next = obj;
                return;
            }
            else
                o = o->m_next;
        }
    }
}

void CTaskBarIcon::RemoveObject(CTaskBarIcon* obj)
{
    if (sm_first == obj)
    {
        sm_first = sm_first->m_next;
    }
    else
    {
        CTaskBarIcon* o = sm_first;
        while (o)
        {
            if (o->m_next == obj)
            {
                o->m_next = obj->m_next;
                return;
            }
            o = o->m_next;
        }
    }
}

BOOL CTaskBarIcon::RegisterWindowClass()
{
    if (sm_registeredClass)
        return TRUE;

    // Also register the taskbar message here
    sm_taskbarMsg = ::RegisterWindowMessage("CtaskBarIconMessage");

    WNDCLASS	wc;
    BOOL	rc;

    HINSTANCE hInstance = GetModuleHandle(NULL);

    /*
     * set up and register window class
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (WNDPROC) CTaskBarIconWindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = 0;
    wc.hCursor = 0;
    wc.hbrBackground = 0;
    wc.lpszMenuName = NULL;
    wc.lpszClassName = CTaskBarWindowClass ;
    rc = ::RegisterClass( &wc );

    sm_registeredClass = (rc != 0);

    return( (rc != 0) );
}

HWND CTaskBarIcon::CreateTaskBarWindow()
{
    HINSTANCE hInstance = GetModuleHandle(NULL);

    HWND hWnd = CreateWindowEx (0, CTaskBarWindowClass,
            "CTaskBarWindow",
            WS_OVERLAPPED,
            0,
            0,
            10,
            10,
            NULL,
            (HMENU) 0,
            hInstance,
            NULL);

    DWORD err = GetLastError();

    return hWnd;
}

long CTaskBarIcon::WindowProc( HWND hWnd, unsigned int msg, unsigned int wParam, long lParam )
{
	if( msg == WM_TASKBAR_EVENT ) {
		Event(wParam);

		return DefWindowProc(hWnd, msg, wParam, lParam);
	}

    if (msg != sm_taskbarMsg)
        return DefWindowProc(hWnd, msg, wParam, lParam);

    switch (lParam)
    {
	case WM_LBUTTONDOWN:
		OnLButtonDown();
		break;

	case WM_LBUTTONUP:
		OnLButtonUp();
		break;

	case WM_RBUTTONDOWN:
		OnRButtonDown();
		break;

	case WM_RBUTTONUP:
		OnRButtonUp();
		break;

	case WM_LBUTTONDBLCLK:
		OnLButtonDClick();
		break;

	case WM_RBUTTONDBLCLK:
		OnRButtonDClick();
		break;

	case WM_MOUSEMOVE:
		OnMouseMove();
		break;

	default:
        break;
	}
    return 0;
}

LONG _EXPORT FAR PASCAL CTaskBarIconWindowProc( HWND hWnd, unsigned msg,
				     UINT wParam, LONG lParam )
{
    CTaskBarIcon* obj = CTaskBarIcon::FindObjectForHWND(hWnd);
    if (obj)
        return obj->WindowProc(hWnd, msg, wParam, lParam);
    else
        return DefWindowProc(hWnd, msg, wParam, lParam);
}

bool CTaskBarIcon::SetUpMainIcon()
{
	if(m_hMainIcon) {
		SetIcon(m_hMainIcon, "Hostle Chat");
		return true;
	}
	else
		return false;
}

void CTaskBarIcon::Event(UINT uiType)
{
	//      ,
	if(m_bEventIncomingMessage || m_pIconFlashThread) 
	{
		if(!(m_uiEventType == EVENT_NEW_USER && uiType == EVENT_INCOMING_MESSAGE))
			return;
		else {
			m_pIconFlashThread->KillThread();
			FinishEvent();
		}
	}

	m_uiEventType = uiType;

	switch(uiType) 
	{
	case EVENT_INCOMING_MESSAGE:
			m_pIconFlashThread = new CIconFlashThread(m_hIncomingMessage, m_hNone,
				TIME_INFINITE, "Incoming Message", "Dos");
		break;
	case EVENT_NEW_USER:
			m_pIconFlashThread = new CIconFlashThread(m_hNewUser, m_hMainIcon,
				6, "New User", "Dos");
		break;
	default:
		return;
	}

	if(!m_pIconFlashThread->CreateThread(CREATE_SUSPENDED)) {
		AfxMessageBox("  .");
		return;
	}
	m_pIconFlashThread->SetThreadPriority(THREAD_PRIORITY_LOWEST);
	m_pIconFlashThread->ResumeThread();

	// ,   
	m_bEventIncomingMessage = true;
}

void CTaskBarIcon::FinishEvent()
{
	if(m_pIconFlashThread) {
		m_pIconFlashThread = NULL;
	}
	SetUpMainIcon();

	m_bEventIncomingMessage = false;
}
