/////////////////////////////////////////////////////////////////////////
// File:      taskbar.h
// Purpose:	  Defines CTaskBarIcon class for manipulating icons on task bar.
// Author:    Julian Smart
// Created:   24/3/98
/////////////////////////////////////////////////////////////////////////

#ifndef _TASKBAR_H_
#define _TASKBAR_H_

#include "IconFlashThread.h"

class CTaskBarIcon
{
	friend class CIconFlashThread;

public:

public:
	CTaskBarIcon(void);
	virtual ~CTaskBarIcon(void);

// Accessors
    inline HWND GetHWND() const { return m_hWnd; }
    inline BOOL IsOK() const { return (m_hWnd != 0) ; }
    inline BOOL IsIconInstalled() const { return m_iconAdded; }

// Operations
    BOOL SetIcon(HICON hIcon, const char* tooltip = "");
    BOOL RemoveIcon(void);

// Overridables
    virtual void OnMouseMove(void);
    virtual void OnLButtonDown(void);
    virtual void OnLButtonUp(void);
    virtual void OnRButtonDown(void);
    virtual void OnRButtonUp(void);
    virtual void OnLButtonDClick(void);
    virtual void OnRButtonDClick(void);

// Implementation
public:
	void FinishEvent(void);
	bool SetUpMainIcon(void);
    long WindowProc( HWND hWnd, unsigned int msg, unsigned int wParam, long lParam );
    static CTaskBarIcon* FindObjectForHWND(HWND hWnd);

	bool m_bEventIncomingMessage;
	CIconFlashThread * m_pIconFlashThread;

    static unsigned int     sm_taskbarMsg;

protected:
    static void AddObject(CTaskBarIcon* obj);
    static void RemoveObject(CTaskBarIcon* obj);
    static BOOL RegisterWindowClass();
    static HWND CreateTaskBarWindow();

	void Event(UINT);

// Data members
protected:
    HWND                    m_hWnd;
    BOOL                    m_iconAdded;
    static BOOL             sm_registeredClass;
    static CTaskBarIcon*    sm_first;
    CTaskBarIcon*           m_next;
private:
	UINT m_uiEventType;
	HICON m_hNewUser;
//	bool m_bEventIncomingMessage;
//	CIconFlashThread * m_pIconFlashThread;
	HICON m_hNone;
	HICON m_hIncomingMessage;
	HICON m_hMainIcon;
};

#endif
    // _TASKBAR_H_

